package com.voice.bdv;

import java.io.IOException;
import java.io.InputStream;
import java.util.Random;

import org.json.JSONArray;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.NinePatch;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.NinePatchDrawable;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.preference.PreferenceManager;
import android.speech.RecognitionListener;
import android.speech.SpeechRecognizer;
import android.util.Log;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.baidu.speech.VoiceRecognitionService;
import com.voice.bdv.UDPWorker.RecvListener;

public class MainActivity extends Activity implements RecognitionListener,
		RecvListener {

	TextView textView;
	ClipCircleView circleView;
	SpeechRecognizer speechRecognizer;
	private static final int EVENT_ERROR = 11;
	UDPWorker send;
	UDPWorker recv;
	String ip;
	String port;
	static final int MSG_DELAY = 100;
	Random rand = new Random();
	Callback callback = new Callback() {

		@Override
		public boolean handleMessage(Message msg) {
			// TODO Auto-generated method stub
			float f = (Float) msg.obj;
			if (f >= 1000) {
				int v = (int) (f / 1000f);
				circleView.initPath(rand.nextInt(v));
				circleView.invalidate();
			}
			return false;
		}
	};
	Handler circleHandler = new Handler(callback);

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		circleView = (ClipCircleView) findViewById(R.id.circle_view);
		circleView.init();
		textView = (TextView) findViewById(R.id.text);
		textView.setText("˵");
		final RelativeLayout layout = (RelativeLayout) findViewById(R.id.bg_layout);
		layout.setBackgroundDrawable(getDrawable(this, "voice_bg.9.png"));
		speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this,
				new ComponentName(this, VoiceRecognitionService.class));
		speechRecognizer.setRecognitionListener(this);
		SharedPreferences preferences = PreferenceManager
				.getDefaultSharedPreferences(this);
		ip = preferences.getString("address", "127.0.0.1");
		port = preferences.getString("port", "4567");
	}

	public static Bitmap getDrawable(Resources res, TypedValue value,
			InputStream stream, Rect rect, BitmapFactory.Options options) {
		if (options == null)
			options = new BitmapFactory.Options();
		if ((options.inDensity == 0) && (value != null)) {
			int i = value.density;
			if (i == 0)
				options.inDensity = 160;
			else if (i != 65535)
				options.inDensity = i;
		}
		if ((options.inTargetDensity == 0) && (res != null))
			options.inTargetDensity = res.getDisplayMetrics().densityDpi;
		return BitmapFactory.decodeStream(stream, rect, options);
	}

	private static Drawable getDrawable(Resources res, Bitmap bitmap,
			byte[] paramArrayOfByte, Rect rect, String path) {
		if (paramArrayOfByte != null)
			return new NinePatchDrawable(res, bitmap, paramArrayOfByte, rect,
					path);
		return new BitmapDrawable(res, bitmap);
	}

	public static Drawable getDrawable(Resources res, TypedValue value,
			InputStream stream, String path, BitmapFactory.Options options) {
		if (stream == null)
			return null;
		Rect rect = new Rect();
		if (options == null)
			options = new BitmapFactory.Options();
		Bitmap bitmap = getDrawable(res, value, stream, rect, options);
		if (bitmap != null) {
			byte[] arrayOfByte = bitmap.getNinePatchChunk();
			if ((arrayOfByte == null)
					|| (!NinePatch.isNinePatchChunk(arrayOfByte))) {
				arrayOfByte = null;
				rect = null;
			}
			return getDrawable(res, bitmap, arrayOfByte, rect, path);
		}
		return null;
	}

	public static Drawable getDrawable(Context context, String path) {
		try {
			Resources res = context.getResources();
			InputStream stream = context.getAssets().open(path);
			TypedValue value = new TypedValue();
			value.density = 240;
			if (Build.VERSION.SDK_INT > 3) {
				return getDrawable(res, value, stream, path, null);
			}
			return Drawable.createFromResourceStream(res, value, stream, path);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		int id = item.getItemId();
		if (id == R.id.action_settings) {
			Intent intent = new Intent(this, SettingsActivity.class);
			startActivityForResult(intent, 1);
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		// TODO Auto-generated method stub
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == 1 && resultCode == 2) {
			SharedPreferences preferences = PreferenceManager
					.getDefaultSharedPreferences(this);
			ip = preferences.getString("address", "127.0.0.1");
			port = preferences.getString("port", "4567");
			if (send != null)
				send.close();
			send = null;
			if (recv != null)
				recv.close();
			recv = null;
		}
	}

	public void onClick(View view) {
		start();
	}

	@Override
	protected void onDestroy() {
		speechRecognizer.destroy();
		super.onDestroy();
		handler.postDelayed(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				System.exit(0);
			}
		}, 1000);
	}

	private Handler handler = new Handler();

	@Override
	public void finish() {
		// TODO Auto-generated method stub
		super.finish();
		speechRecognizer.stopListening();
	}

	private long startTime;

	private void start() {
		if (send == null)
			send = new UDPWorker(this, ip, port);
		if (recv == null) {
			recv = new UDPWorker(this, port);
			recv.setRecvListener(this);
		}
		Intent intent = new Intent();
		intent.putExtra("args", "");
		speechRecognizer.startListening(intent);
		startTime = System.currentTimeMillis();
		textView.setText("");
	}

	@Override
	public void onReadyForSpeech(Bundle params) {
		// TODO Auto-generated method stub
	}

	@Override
	public void onBeginningOfSpeech() {
		// TODO Auto-generated method stub
		// print("⵽ûѾʼ˵");
		textView.setText("");
	}

	@Override
	public void onRmsChanged(float rmsdB) {
		// TODO Auto-generated method stub
		Message msg = new Message();
		msg.obj = Float.valueOf(rmsdB);
		circleHandler.sendMessage(msg);
		long now = System.currentTimeMillis();
		long time = now - startTime;
		Log.d("======", time + "===" + rmsdB);
		if (time > 3000 && rmsdB < 1000) {
			speechRecognizer.stopListening();
		} else if (time > 8000) {
			speechRecognizer.stopListening();
		}
	}

	@Override
	public void onBufferReceived(byte[] buffer) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onEndOfSpeech() {
		// TODO Auto-generated method stub
		// print("⵽ûѾֹͣ˵");
		textView.setText("ʶ");
		circleView.initPath(0);
		circleView.invalidate();
	}

	@Override
	public void onError(int error) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onResults(Bundle results) {
		// TODO Auto-generated method stub
		String json_res = results.getString("origin_result");
		try {
			JSONObject root = new JSONObject(json_res);
			JSONObject content = root.getJSONObject("content");
			JSONArray item = content.getJSONArray("item");
			if (item.length() <= 0)
				return;
			String code = item.getString(0);
			if (send != null)
				send.sendMsg(code);
			showMsg(code);
			textView.setText("˵");
		} catch (Exception e) {
		}
	}

	@Override
	public void onPartialResults(Bundle partialResults) {
		// TODO Auto-generated method stub
	}

	@Override
	public void onEvent(int eventType, Bundle params) {
		// TODO Auto-generated method stub
		switch (eventType) {
		case EVENT_ERROR:
			showMsg("δ⵽");
			textView.setText("˵");
			break;
		case VoiceRecognitionService.EVENT_ENGINE_SWITCH:
			// int type = params.getInt("engine_type");
			// print("*л" + (type == 0 ? "" : ""));
			break;
		}
	}

	@Override
	public void onRecv(byte[] data, int offset, int length) {
		// TODO Auto-generated method stub
		// final String text = new String(data, offset, length);
		// showMsg("recv:" + text);
	}

	private void showMsg(final String text) {
		Message msg = new Message();
		msg.obj = text;
		msgHandler.sendMessage(msg);
	}

	private Callback msgCallback = new Callback() {

		@SuppressLint("ShowToast")
		@Override
		public boolean handleMessage(Message msg) {
			// TODO Auto-generated method stub
			if (msgToast == null)
				msgToast = Toast.makeText(MainActivity.this, "",
						Toast.LENGTH_LONG);
			msgToast.setText(msg.obj.toString());
			msgToast.show();
			return false;
		}
	};
	private Toast msgToast;
	private Handler msgHandler = new Handler(msgCallback);

}
