package com.voice.bdv;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PaintFlagsDrawFilter;
import android.graphics.Path;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.View;

public class ClipCircleView extends View {

	private Drawable empty;
	private Drawable full;
	Path path;
	PaintFlagsDrawFilter filter;

	public ClipCircleView(Context paramContext, AttributeSet attrs) {
		super(paramContext, attrs);
		setLayerType(View.LAYER_TYPE_SOFTWARE, null);// 

	}

	public void init() {
		try {
			empty = MainActivity.getDrawable(getContext(), "voice_empty.png");
			full = MainActivity.getDrawable(getContext(), "voice_full.png");
			this.empty.setBounds(new Rect(-this.empty.getIntrinsicWidth() / 2,
					-this.empty.getIntrinsicHeight() / 2, this.empty
							.getIntrinsicWidth() / 2, this.empty
							.getIntrinsicHeight() / 2));
			this.full.setBounds(new Rect(-this.full.getIntrinsicWidth() / 2,
					-this.full.getIntrinsicHeight() / 2, this.full
							.getIntrinsicWidth() / 2, this.full
							.getIntrinsicHeight() / 2));
			filter = new PaintFlagsDrawFilter(Paint.ANTI_ALIAS_FLAG,
					Paint.FILTER_BITMAP_FLAG);
			path = new Path();
			initPath(0);
		} catch (Exception localException) {
			localException.printStackTrace();
		}
	}

	public void initPath(int paramInt) {
		this.path.reset();
		this.path.addCircle(0.0F, 0.0F, this.empty.getIntrinsicWidth()
				* paramInt / 12, Path.Direction.CCW);
	}

	public void onDraw(Canvas paramCanvas) {
		paramCanvas.save();
		paramCanvas.setDrawFilter(filter);
		paramCanvas.translate(getWidth() / 2, getHeight() / 2);
		if (empty != null)
			empty.draw(paramCanvas);
		if (path != null)
			paramCanvas.clipPath(path);
		if (full != null)
			full.draw(paramCanvas);
		paramCanvas.restore();
	}

	public void finalize() throws Throwable {
		this.empty = null;
		this.full = null;
		super.finalize();
	}

	protected void onMeasure(int paramInt1, int paramInt2) {
		super.onMeasure(paramInt1, paramInt2);
		int i = View.MeasureSpec.getSize(paramInt1);
		int j = View.MeasureSpec.getSize(paramInt2);
		Drawable localDrawable = getBackground();
		if (localDrawable != null) {
			i = localDrawable.getMinimumWidth();
			j = localDrawable.getMinimumHeight();
		}
		setMeasuredDimension(resolveSize(i, paramInt1),
				resolveSize(j, paramInt2));
	}
}
